@@ -24,10 +24,10 @@ cat "$dockerdir"/Dockerfile_test_appimage.in | \
     perl -p -e "s/\[\[REVISION\]\]/$current/g" > \
          "$dockerdir"/Dockerfile_test_appimage.gen
 
-grep '^bitbucket.org' ~/.ssh/known_hosts > "$dockerdir"/known_hosts
-cp ~/.ssh/id_dsa_build "$dockerdir"/id_dsa_build
-chmod 600 "$dockerdir"/known_hosts "$dockerdir"/id_dsa_build
-trap "rm $dockerdir/known_hosts $dockerdir/id_dsa_build" 0
+fgrep 'hg.sr.ht' ~/.ssh/known_hosts > "$dockerdir"/known_hosts
+cp ~/.ssh/id_rsa_build "$dockerdir"/id_rsa_build
+chmod 600 "$dockerdir"/known_hosts "$dockerdir"/id_rsa_build
+trap "rm $dockerdir/known_hosts $dockerdir/id_rsa_build" 0
 
 dockertag="cannam/sonic-visualiser-appimage-$current"
 
