@@ -31,7 +31,7 @@
 
 #include <QMetaType>
 #include <QApplication>
-#include <QDesktopWidget>
+#include <QScreen>
 #include <QMessageBox>
 #include <QTranslator>
 #include <QLocale>
@@ -42,6 +42,7 @@
 #include <QTimer>
 #include <QPainter>
 #include <QFileOpenEvent>
+#include <QCommandLineParser>
 
 #include <iostream>
 #include <signal.h>
@@ -90,6 +91,8 @@
 
 \section model Data sources: the Model hierarchy
 
+***!!! todo: update this
+
    A Model is something containing, or knowing how to obtain, data.
 
    For example, WaveFileModel is a model that knows how to get data
@@ -185,12 +188,15 @@ signalHandler(int /* signal */)
     // Avoid this happening more than once across threads
 
     cerr << "signalHandler: cleaning up and exiting" << endl;
-    cleanupMutex.lock();
-    if (!cleanedUp) {
-        TempDirectory::getInstance()->cleanup();
-        cleanedUp = true;
+
+    if (cleanupMutex.tryLock(5000)) {
+        if (!cleanedUp) {
+            TempDirectory::getInstance()->cleanup();
+            cleanedUp = true;
+        }
+        cleanupMutex.unlock();
     }
-    cleanupMutex.unlock();
+    
     exit(0);
 }
 
@@ -201,13 +207,13 @@ public:
         QApplication(argc, argv),
         m_readyForFiles(false),
         m_filepathQueue(QStringList()),
-        m_mainWindow(0)
+        m_mainWindow(nullptr)
     {
     }
-    virtual ~SVApplication() { }
+    ~SVApplication() override { }
 
     void setMainWindow(MainWindow *mw) { m_mainWindow = mw; }
-    void releaseMainWindow() { m_mainWindow = 0; }
+    void releaseMainWindow() { m_mainWindow = nullptr; }
 
     virtual void commitData(QSessionManager &manager) {
         if (!m_mainWindow) return;
@@ -224,7 +230,7 @@ public:
 
 protected:
     MainWindow *m_mainWindow;
-    bool event(QEvent *);
+    bool event(QEvent *) override;
 };
 
 int
@@ -235,25 +241,73 @@ main(int argc, char **argv)
         cerr << SV_VERSION << endl;
         exit(0);
     }
-
+    
     svSystemSpecificInitialisation();
 
-#ifdef Q_WS_X11
-#if QT_VERSION >= 0x040500
-//    QApplication::setGraphicsSystem("raster");
-#endif
-#endif
-
-#ifdef Q_OS_MAC
-    if (QSysInfo::MacintoshVersion > QSysInfo::MV_10_8) {
-        // Fix for OS/X 10.9 font problem
-        QFont::insertSubstitution(".Lucida Grande UI", "Lucida Grande");
-    }
-#endif
-
     SVApplication application(argc, argv);
 
+    QApplication::setOrganizationName("sonic-visualiser");
+    QApplication::setOrganizationDomain("sonicvisualiser.org");
+    QApplication::setApplicationName(QApplication::tr("Sonic Visualiser"));
+    QApplication::setApplicationVersion(SV_VERSION);
+
+    //!!! todo hand-update translations
+    QCommandLineParser parser;
+    parser.setApplicationDescription(QApplication::tr("\nSonic Visualiser is a program for viewing and exploring audio data\nfor semantic music analysis and annotation."));
+    parser.addHelpOption();
+    parser.addVersionOption();
+
+    parser.addOption(QCommandLineOption
+                     ("no-audio", QApplication::tr
+                      ("Do not attempt to open an audio output device.")));
+    parser.addOption(QCommandLineOption
+                     ("no-osc", QApplication::tr
+                      ("Do not provide an Open Sound Control port for remote control.")));
+    parser.addOption(QCommandLineOption
+                     ("no-splash", QApplication::tr
+                      ("Do not show a splash screen.")));
+    parser.addOption(QCommandLineOption
+                     ("osc-script", QApplication::tr
+                      ("Batch run the Open Sound Control script found in the given file. Supply \"-\" as file to read from stdin. Scripts consist of /command arg1 arg2 ... OSC control lines, optionally interleaved with numbers to specify pauses in seconds."),
+                      "osc.txt"));
+    parser.addOption(QCommandLineOption
+                     ("first-run", QApplication::tr
+                      ("Clear any saved settings and reset to first-run behaviour.")));
+
+    parser.addPositionalArgument
+        ("[<file> ...]", QApplication::tr("One or more Sonic Visualiser (.sv) and audio files may be provided."));
+    
     QStringList args = application.arguments();
+    if (!parser.parse(args)) {
+        if (parser.unknownOptionNames().contains("?")) {
+            // QCommandLineParser only understands -? for help on Windows,
+            // but we historically accepted it everywhere - provide this
+            // backward compatibility
+            parser.showHelp();
+        }
+    }        
+        
+    parser.process(args);
+
+    if (parser.isSet("first-run")) {
+        QSettings settings;
+        settings.clear();
+    }
+
+    bool audioOutput = !(parser.isSet("no-audio"));
+    bool oscSupport = !(parser.isSet("no-osc"));
+    bool showSplash = !(parser.isSet("no-splash"));
+
+    if (!audioOutput) {
+        SVDEBUG << "Note: --no-audio flag set, will not use audio device" << endl;
+    }
+    if (!oscSupport) {
+        SVDEBUG << "Note: --no-osc flag set, will not open OSC port" << endl;
+    }
+    
+    args = parser.positionalArguments();
+
+    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
 
     signal(SIGINT,  signalHandler);
     signal(SIGTERM, signalHandler);
@@ -263,25 +317,7 @@ main(int argc, char **argv)
     signal(SIGQUIT, signalHandler);
 #endif
 
-    bool audioOutput = true;
-    bool oscSupport = true;
-
-    if (args.contains("--help") || args.contains("-h") || args.contains("-?")) {
-        cerr << QApplication::tr(
-            "\nSonic Visualiser is a program for viewing and exploring audio data\nfor semantic music analysis and annotation.\n\nUsage:\n\n  %1 [--no-audio] [--no-osc] [<file> ...]\n\n  --no-audio: Do not attempt to open an audio output device\n  --no-osc: Do not provide an Open Sound Control port for remote control\n  <file>: One or more Sonic Visualiser (.sv) and audio files may be provided.\n").arg(argv[0]) << endl;
-        exit(2);
-    }
-
-    if (args.contains("--no-audio")) audioOutput = false;
-    if (args.contains("--no-osc")) oscSupport = false;
-
-    QApplication::setOrganizationName("sonic-visualiser");
-    QApplication::setOrganizationDomain("sonicvisualiser.org");
-    QApplication::setApplicationName(QApplication::tr("Sonic Visualiser"));
-
-    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-
-    SVSplash *splash = 0;
+    SVSplash *splash = nullptr;
 
     QSettings settings;
 
@@ -293,7 +329,7 @@ main(int argc, char **argv)
     settings.endGroup();
 
     settings.beginGroup("Preferences");
-    if (settings.value("show-splash", true).toBool()) {
+    if (showSplash && settings.value("show-splash", true).toBool()) {
         splash = new SVSplash();
         splash->show();
         QTimer::singleShot(5000, splash, SLOT(hide()));
@@ -361,11 +397,19 @@ main(int argc, char **argv)
     qRegisterMetaType<PropertyContainer::PropertyName>("PropertyContainer::PropertyName");
     qRegisterMetaType<ZoomLevel>("ZoomLevel");
 
-    MainWindow::SoundOptions options = MainWindow::WithEverything;
-    if (!audioOutput) options = 0;
+    MainWindow::AudioMode audioMode = 
+        MainWindow::AUDIO_PLAYBACK_NOW_RECORD_LATER;
+    MainWindow::MIDIMode midiMode =
+        MainWindow::MIDI_LISTEN;
+
+    if (!audioOutput) {
+        audioMode = MainWindow::AUDIO_NONE;
+        midiMode = MainWindow::MIDI_NONE;
+    } 
     
-    MainWindow *gui = new MainWindow(options, oscSupport);
+    MainWindow *gui = new MainWindow(audioMode, midiMode, oscSupport);
     application.setMainWindow(gui);
+
     InteractiveFileFinder::setParentWidget(gui);
     TransformUserConfigurator::setParentWidget(gui);
     if (splash) {
@@ -374,8 +418,8 @@ main(int argc, char **argv)
                          splash, SLOT(finishSplash(QWidget *)));
     }
 
-    QDesktopWidget *desktop = QApplication::desktop();
-    QRect available = desktop->availableGeometry();
+    QScreen *screen = QApplication::primaryScreen();
+    QRect available = screen->availableGeometry();
 
     int width = (available.width() * 2) / 3;
     int height = available.height() / 2;
@@ -411,7 +455,11 @@ main(int argc, char **argv)
 
     for (QStringList::iterator i = args.begin(); i != args.end(); ++i) {
 
-        if (i == args.begin()) continue;
+        // Note QCommandLineParser has now pulled out argv[0] and all
+        // the options, so in theory everything here from the very
+        // first arg should be relevant. But let's reject names
+        // starting with "-" just in case.
+        
         if (i->startsWith('-')) continue;
 
         QString path = *i;
@@ -419,7 +467,8 @@ main(int argc, char **argv)
         application.handleFilepathArgument(path, splash);
     }
     
-    for (QStringList::iterator i = application.m_filepathQueue.begin(); i != application.m_filepathQueue.end(); ++i) {
+    for (QStringList::iterator i = application.m_filepathQueue.begin();
+         i != application.m_filepathQueue.end(); ++i) {
         QString path = *i;
         application.handleFilepathArgument(path, splash);
     }
@@ -439,6 +488,13 @@ main(int argc, char **argv)
     settings.endGroup();
 #endif
 
+    QString scriptFile = parser.value("osc-script");
+    if (scriptFile != "") {
+        SVDEBUG << "Note: Cueing OSC script from filename \"" << scriptFile
+                << "\"" << endl;
+        gui->cueOSCScript(scriptFile);
+    }
+    
     int rv = application.exec();
 
     gui->hide();
@@ -492,7 +548,7 @@ bool SVApplication::event(QEvent *event){
     case QEvent::FileOpen:
         thePath = static_cast<QFileOpenEvent *>(event)->file();
         if(m_readyForFiles)
-            handleFilepathArgument(thePath, NULL);
+            handleFilepathArgument(thePath, nullptr);
         else
             m_filepathQueue.append(thePath);
         return true;
