@@ -46,19 +46,34 @@ Layer::~Layer()
 }
 
 void
-Layer::connectSignals(const Model *model)
+Layer::connectSignals(ModelId modelId)
 {
-    connect(model, SIGNAL(modelChanged()),
-            this, SIGNAL(modelChanged()));
+    auto model = ModelById::get(modelId);
+    if (!model) return;
+    
+    connect(model.get(), SIGNAL(modelChanged(ModelId)),
+            this, SIGNAL(modelChanged(ModelId)));
 
-    connect(model, SIGNAL(modelChangedWithin(sv_frame_t, sv_frame_t)),
-            this, SIGNAL(modelChangedWithin(sv_frame_t, sv_frame_t)));
+    connect(model.get(), SIGNAL(modelChangedWithin(ModelId, sv_frame_t, sv_frame_t)),
+            this, SIGNAL(modelChangedWithin(ModelId, sv_frame_t, sv_frame_t)));
 
-    connect(model, SIGNAL(completionChanged()),
-            this, SIGNAL(modelCompletionChanged()));
+    connect(model.get(), SIGNAL(completionChanged(ModelId)),
+            this, SIGNAL(modelCompletionChanged(ModelId)));
 
-    connect(model, SIGNAL(alignmentCompletionChanged()),
-            this, SIGNAL(modelAlignmentCompletionChanged()));
+    connect(model.get(), SIGNAL(alignmentCompletionChanged(ModelId)),
+            this, SIGNAL(modelAlignmentCompletionChanged(ModelId)));
+}
+
+ModelId
+Layer::getSourceModel() const
+{
+    ModelId sourceId;
+    auto model = ModelById::get(getModel());
+    while (model && !model->getSourceModel().isNone()) {
+        sourceId = model->getSourceModel();
+        model = ModelById::get(sourceId);
+    }
+    return sourceId;
 }
 
 QString
@@ -84,7 +99,8 @@ Layer::getLayerPresentationName() const
         (factory->getLayerType(this));
 
     QString modelName;
-    if (getModel()) modelName = getModel()->objectName();
+    auto model = ModelById::get(getModel());
+    if (model) modelName = model->objectName();
             
     QString text;
     if (modelName != "") {
@@ -103,15 +119,11 @@ Layer::setObjectName(const QString &name)
     emit layerNameChanged();
 }
 
-PlayParameters *
+std::shared_ptr<PlayParameters>
 Layer::getPlayParameters() 
 {
-//    cerr << "Layer (" << this << ", " << objectName() << ")::getPlayParameters: model is "<< getModel() << endl;
-    const Model *model = getModel();
-    if (model) {
-        return PlayParameterRepository::getInstance()->getPlayParameters(model);
-    }
-    return 0;
+    return PlayParameterRepository::getInstance()->getPlayParameters
+        (getModel().untyped);
 }
 
 void
@@ -143,10 +155,10 @@ Layer::getXScaleValue(const LayerGeometryProvider *v, int x, double &value, QStr
 {
     if (!hasTimeXAxis()) return false;
 
-    const Model *m = getModel();
-    if (!m) return false;
+    auto model = ModelById::get(getModel());
+    if (!model) return false;
 
-    value = double(v->getFrameForX(x)) / m->getSampleRate();
+    value = double(v->getFrameForX(x)) / model->getSampleRate();
     unit = "s";
     return true;
 }
@@ -168,10 +180,9 @@ Layer::getYScaleDifference(const LayerGeometryProvider *v, int y0, int y1,
 sv_frame_t
 Layer::alignToReference(LayerGeometryProvider *v, sv_frame_t frame) const
 {
-    const Model *m = getModel();
-    SVDEBUG << "Layer::alignToReference(" << frame << "): model = " << m << ", alignment reference = " << (m ? m->getAlignmentReference() : 0) << endl;
-    if (m && m->getAlignmentReference()) {
-        return m->alignToReference(frame);
+    auto model = ModelById::get(getModel());
+    if (model && !model->getAlignmentReference().isNone()) {
+        return model->alignToReference(frame);
     } else {
         return v->getView()->alignToReference(frame);
     }
@@ -180,10 +191,9 @@ Layer::alignToReference(LayerGeometryProvider *v, sv_frame_t frame) const
 sv_frame_t
 Layer::alignFromReference(LayerGeometryProvider *v, sv_frame_t frame) const
 {
-    const Model *m = getModel();
-    SVDEBUG << "Layer::alignFromReference(" << frame << "): model = " << m << ", alignment reference = " << (m ? m->getAlignmentReference() : 0) << endl;
-    if (m && m->getAlignmentReference()) {
-        return m->alignFromReference(frame);
+    auto model = ModelById::get(getModel());
+    if (model && !model->getAlignmentReference().isNone()) {
+        return model->alignFromReference(frame);
     } else {
         return v->getView()->alignFromReference(frame);
     }
@@ -232,7 +242,7 @@ Layer::clipboardHasDifferentAlignment(LayerGeometryProvider *v, const Clipboard
     // We either want to be literal all the way through, or aligned
     // all the way through.
 
-    for (Clipboard::PointList::const_iterator i = clip.getPoints().begin();
+    for (EventVector::const_iterator i = clip.getPoints().begin();
          i != clip.getPoints().end(); ++i) {
 
         // In principle, we want to know whether the aligned version
@@ -252,12 +262,12 @@ Layer::clipboardHasDifferentAlignment(LayerGeometryProvider *v, const Clipboard
         
         sv_frame_t sourceFrame = i->getFrame();
         sv_frame_t referenceFrame = sourceFrame;
-        if (i->haveReferenceFrame()) {
+        if (i->hasReferenceFrame()) {
             referenceFrame = i->getReferenceFrame();
         }
         sv_frame_t myMappedFrame = alignToReference(v, sourceFrame);
 
-//        cerr << "sourceFrame = " << sourceFrame << ", referenceFrame = " << referenceFrame << " (have = " << i->haveReferenceFrame() << "), myMappedFrame = " << myMappedFrame << endl;
+//        cerr << "sourceFrame = " << sourceFrame << ", referenceFrame = " << referenceFrame << " (have = " << i->hasReferenceFrame() << "), myMappedFrame = " << myMappedFrame << endl;
 
         if (myMappedFrame != referenceFrame) return true;
     }
@@ -373,8 +383,24 @@ Layer::DeleteMeasurementRectCommand::unexecute()
 void
 Layer::measureStart(LayerGeometryProvider *v, QMouseEvent *e)
 {
-    setMeasureRectFromPixrect(v, m_draggingRect,
-                              QRect(e->x(), e->y(), 0, 0));
+    m_draggingRect.haveFrames = hasTimeXAxis();
+
+    // NB if haveFrames, then pixrect x and width will be rewritten on
+    // every paint according to the current locations of the
+    // definitive frame values. So we should set the start frame value
+    // once on measureStart, and then not modify it on drag (to avoid
+    // drift from repeated conversion back and forth).
+    
+    m_draggingRect.pixrect = QRect(e->x(), e->y(), 0, 0);
+
+    if (m_draggingRect.haveFrames) {
+        m_draggingRect.startFrame = v->getFrameForX(e->x());
+        m_draggingRect.endFrame = v->getFrameForX(e->x());
+    }
+
+    setMeasureRectYCoord(v, m_draggingRect, true, e->y());
+    setMeasureRectYCoord(v, m_draggingRect, false, e->y());
+
     m_haveDraggingRect = true;
 }
 
@@ -383,11 +409,15 @@ Layer::measureDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
     if (!m_haveDraggingRect) return;
 
-    setMeasureRectFromPixrect(v, m_draggingRect,
-                              QRect(m_draggingRect.pixrect.x(),
-                                    m_draggingRect.pixrect.y(),
-                                    e->x() - m_draggingRect.pixrect.x(),
-                                    e->y() - m_draggingRect.pixrect.y()));
+    m_draggingRect.pixrect.setHeight(e->y() - m_draggingRect.pixrect.y());
+
+    if (m_draggingRect.haveFrames) {
+        m_draggingRect.endFrame = v->getFrameForX(e->x());
+    } else {
+        m_draggingRect.pixrect.setWidth(e->x() - m_draggingRect.pixrect.x());
+    }
+
+    setMeasureRectYCoord(v, m_draggingRect, false, e->y());
 }
 
 void
@@ -600,7 +630,7 @@ Layer::valueExtentsMatchMine(LayerGeometryProvider *v) const
         return false;
     }
 
-    if (!v->getValueExtents(unit, min, max, logarithmic)) {
+    if (!v->getVisibleExtentsForUnit(unit, min, max, logarithmic)) {
         return false;
     }
 
@@ -624,12 +654,16 @@ Layer::toXml(QTextStream &stream,
             .arg(extraAttributes).arg(encodeEntities(m_presentationName));
     }
 
+    int modelExportId = -1;
+    auto model = ModelById::get(getModel());
+    if (model) modelExportId = model->getExportId();
+    
     stream << QString("<layer id=\"%2\" type=\"%1\" name=\"%3\" model=\"%4\" %5")
         .arg(encodeEntities(LayerFactory::getInstance()->getLayerTypeName
                             (LayerFactory::getInstance()->getLayerType(this))))
-        .arg(getObjectExportId(this))
+        .arg(getExportId())
         .arg(encodeEntities(objectName()))
-        .arg(getObjectExportId(getModel()))
+        .arg(modelExportId)
         .arg(extraAttributes);
 
     if (m_measureRects.empty()) {
@@ -658,12 +692,16 @@ Layer::toBriefXml(QTextStream &stream,
             .arg(extraAttributes).arg(encodeEntities(m_presentationName));
     }
 
+    int modelExportId = -1;
+    auto model = ModelById::get(getModel());
+    if (model) modelExportId = model->getExportId();
+
     stream << QString("<layer id=\"%2\" type=\"%1\" name=\"%3\" model=\"%4\" %5/>\n")
         .arg(encodeEntities(LayerFactory::getInstance()->getLayerTypeName
                             (LayerFactory::getInstance()->getLayerType(this))))
-        .arg(getObjectExportId(this))
+        .arg(getExportId())
         .arg(encodeEntities(objectName()))
-        .arg(getObjectExportId(getModel()))
+        .arg(modelExportId)
         .arg(extraAttributes);
 }
 
