@@ -35,6 +35,7 @@
 
 //!!! ugh
 #include "data/model/WaveFileModel.h"
+#include "data/model/AlignmentModel.h"
 
 #include <QPaintEvent>
 #include <QPainter>
@@ -60,10 +61,11 @@
 
 #include "widgets/KeyReference.h" //!!! should probably split KeyReference into a data class in base and another that shows the widget
 
-//#define DEBUG_PANE
+//#define DEBUG_PANE 1
+//#define DEBUG_PANE_SCALE_CHOICE 1
 
-QCursor *Pane::m_measureCursor1 = 0;
-QCursor *Pane::m_measureCursor2 = 0;
+QCursor *Pane::m_measureCursor1 = nullptr;
+QCursor *Pane::m_measureCursor2 = nullptr;
 
 Pane::Pane(QWidget *w) :
     View(w, true),
@@ -79,11 +81,11 @@ Pane::Pane(QWidget *w) :
     m_centreLineVisible(true),
     m_scaleWidth(0),
     m_pendingWheelAngle(0),
-    m_headsUpDisplay(0),
-    m_vpan(0),
-    m_hthumb(0),
-    m_vthumb(0),
-    m_reset(0),
+    m_headsUpDisplay(nullptr),
+    m_vpan(nullptr),
+    m_hthumb(nullptr),
+    m_vthumb(nullptr),
+    m_reset(nullptr),
     m_mouseInWidget(false),
     m_playbackFrameMoveScheduled(false),
     m_playbackFrameMoveTo(0)
@@ -107,7 +109,7 @@ Pane::updateHeadsUpDisplay()
 
     if (!isVisible()) return;
 
-    Layer *layer = 0;
+    Layer *layer = nullptr;
     if (getLayerCount() > 0) {
         layer = getLayer(getLayerCount() - 1);
     }
@@ -252,7 +254,7 @@ Pane::updateVerticalPanner()
     // implement vertical zoom steps as well, but they don't all at
     // the moment.
 
-    Layer *layer = 0;
+    Layer *layer = nullptr;
     if (getLayerCount() > 0) layer = getLayer(getLayerCount() - 1);
     int discard;
     if (layer && layer->getVerticalZoomSteps(discard) == 0) {
@@ -367,33 +369,46 @@ Pane::paintEvent(QPaintEvent *e)
     Layer *topLayer = getTopLayer();
     bool haveSomeTimeXAxis = false;
 
-    const Model *waveformModel = 0; // just for reporting purposes
-    const Model *workModel = 0;
+    ModelId waveformModelId; // just for reporting purposes
+    ModelId workModelId;
+
+    for (LayerList::iterator vi = m_layerStack.end();
+         vi != m_layerStack.begin(); ) {
 
-    for (LayerList::iterator vi = m_layerStack.end(); vi != m_layerStack.begin(); ) {
         --vi;
+
         if (!haveSomeTimeXAxis && (*vi)->hasTimeXAxis()) {
             haveSomeTimeXAxis = true;
         }
-        if (dynamic_cast<WaveformLayer *>(*vi)) {
-            waveformModel = (*vi)->getModel();
-            workModel = waveformModel;
-        } else {
-            Model *m = (*vi)->getModel();
-            if (dynamic_cast<WaveFileModel *>(m)) {
-                workModel = m;
-            } else if (m && dynamic_cast<WaveFileModel *>(m->getSourceModel())) {
-                workModel = m->getSourceModel();
+
+        ModelId modelId = (*vi)->getModel();
+        if (!modelId.isNone()) {
+            if (dynamic_cast<WaveformLayer *>(*vi)) {
+                waveformModelId = modelId;
+                workModelId = modelId;
+            } else {
+                if (ModelById::isa<WaveFileModel>(modelId)) {
+                    workModelId = modelId;
+                } else {
+                    ModelId sourceId = (*vi)->getSourceModel();
+                    if (ModelById::isa<WaveFileModel>(sourceId)) {
+                        workModelId = sourceId;
+                    }
+                }
             }
         }
                 
-        if (waveformModel && workModel && haveSomeTimeXAxis) break;
+        if (!waveformModelId.isNone() &&
+            !workModelId.isNone() &&
+            haveSomeTimeXAxis) {
+            break;
+        }
     }
 
     // Block off left and right extents so we can see where the main model ends
     
-    if (workModel && hasTopLayerTimeXAxis()) {
-        drawModelTimeExtents(r, paint, workModel);
+    if (!workModelId.isNone() && hasTopLayerTimeXAxis()) {
+        drawModelTimeExtents(r, paint, workModelId);
     }
 
     // Crosshairs for mouse movement in measure mode
@@ -444,26 +459,26 @@ Pane::paintEvent(QPaintEvent *e)
     
     paint.setPen(QColor(50, 50, 50));
 
-    if (waveformModel &&
+    if (!waveformModelId.isNone() &&
         sampleRate &&
         m_manager &&
         m_manager->shouldShowDuration()) {
-        drawDurationAndRate(r, waveformModel, sampleRate, paint);
+        drawDurationAndRate(r, waveformModelId, sampleRate, paint);
     }
 
     bool haveWorkTitle = false;
 
-    if (workModel &&
+    if (!workModelId.isNone() &&
         m_manager &&
         m_manager->shouldShowWorkTitle()) {
-        drawWorkTitle(r, paint, workModel);
+        drawWorkTitle(r, paint, workModelId);
         haveWorkTitle = true;
     }
 
-    if (workModel &&
+    if (!workModelId.isNone() &&
         m_manager &&
         m_manager->getAlignMode()) {
-        drawAlignmentStatus(r, paint, workModel, haveWorkTitle);
+        drawAlignmentStatus(r, paint, workModelId, haveWorkTitle);
     }
 
     if (m_manager &&
@@ -511,97 +526,141 @@ Pane::getVerticalScaleWidth() const
 void
 Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
 {
-    Layer *scaleLayer = 0;
-
-//    cerr << "Pane::drawVerticalScale[" << this << "]" << endl;
-    
     double min, max;
     bool log;
     QString unit;
 
-    // If the top layer has no scale and reports no display
-    // extents, but does report a unit, then the scale should be
-    // drawn from any underlying layer with a scale and that unit.
-    // If the top layer has no scale and no value extents at all,
-    // then the scale should be drawn from any underlying layer
-    // with a scale regardless of unit.
-
-    int sw = topLayer->getVerticalScaleWidth
-        (this, m_manager->shouldShowVerticalColourScale(), paint);
+    bool includeColourScale = m_manager->shouldShowVerticalColourScale();
     
-    if (sw > 0) {
-        scaleLayer = topLayer;
-        m_scaleWidth = sw;
+    Layer *scaleLayer = nullptr;
+    int scaleWidth = 0;
 
+#ifdef DEBUG_PANE_SCALE_CHOICE
+        SVCERR << "Pane[" << getId() << "]::drawVerticalScale: Have "
+               << getLayerCount() << " layer(s)" << endl;
+#endif
+
+    // If the topmost layer is prepared to draw a scale, then use it.
+    //
+    // Otherwise: find the topmost layer that has value extents,
+    // i.e. for which a scale is relevant at all.
+    //
+    // If that layer is prepared to draw a scale directly, then use
+    // it. This could be the case even if the layer has no unit and so
+    // does not participate in scale-providing / auto-align layers.
+    // 
+    // Otherwise, request the scale-providing layer for that layer
+    // from the view, and if there is one and it can draw a scale, use
+    // that.
+    //
+    // In all cases ignore dormant layers, and if we hit an opaque
+    // layer before finding any with value extents, give up.
+
+    if (topLayer && !topLayer->isLayerDormant(this)) {
+        scaleWidth = topLayer->getVerticalScaleWidth
+            (this, includeColourScale, paint);
+
+#ifdef DEBUG_PANE_SCALE_CHOICE
+        SVCERR << "Pane[" << getId() << "]::drawVerticalScale: Top layer ("
+               << topLayer << ", " << topLayer->getLayerPresentationName()
+               << ") offers vertical scale width of " << scaleWidth
+               << endl;
+#endif
+    }
+
+    if (scaleWidth > 0) {
+        scaleLayer = topLayer;
+
+#ifdef DEBUG_PANE_SCALE_CHOICE
+        SVCERR << "Pane[" << getId() << "]::drawVerticalScale: Accepting that"
+               << endl;
+#endif
     } else {
 
-        bool hasDisplayExtents = topLayer->getDisplayExtents(min, max);
-        bool hasValueExtents = topLayer->getValueExtents(min, max, log, unit);
-            
-        if (!hasDisplayExtents) {
+        for (auto i = m_layerStack.rbegin(); i != m_layerStack.rend(); ++i) {
+            Layer *layer = *i;
 
-            if (!hasValueExtents) {
-
-                for (LayerList::iterator vi = m_layerStack.end();
-                     vi != m_layerStack.begin(); ) {
-                        
-                    --vi;
-                        
-                    if ((*vi) == topLayer) continue;
-                        
-                    sw = (*vi)->getVerticalScaleWidth
-                        (this, m_manager->shouldShowVerticalColourScale(), paint);
-                        
-                    if (sw > 0) {
-                        scaleLayer = *vi;
-                        m_scaleWidth = sw;
-                        break;
-                    }
-                }
-            } else if (unit != "") { // && hasValueExtents && !hasDisplayExtents
-
-                QString requireUnit = unit;
-
-                for (LayerList::iterator vi = m_layerStack.end();
-                     vi != m_layerStack.begin(); ) {
-                        
-                    --vi;
-                        
-                    if ((*vi) == topLayer) continue;
-                        
-                    if ((*vi)->getDisplayExtents(min, max)) {
-                            
-                        // search no further than this: if the
-                        // scale from this layer isn't suitable,
-                        // we'll have to draw no scale (else we'd
-                        // risk ending up with the wrong scale)
-                            
-                        if ((*vi)->getValueExtents(min, max, log, unit) &&
-                            unit == requireUnit) {
-
-                            sw = (*vi)->getVerticalScaleWidth
-                                (this, m_manager->shouldShowVerticalColourScale(), paint);
-                            if (sw > 0) {
-                                scaleLayer = *vi;
-                                m_scaleWidth = sw;
-                            }
-                        }
-                        break;
-                    }
-                }
+            if (layer->isLayerDormant(this)) {
+#ifdef DEBUG_PANE_SCALE_CHOICE
+                SVCERR << "Pane[" << getId() << "]::drawVerticalScale: "
+                       << "Layer " << layer << ", "
+                       << layer->getLayerPresentationName()
+                       << " is dormant, skipping" << endl;
+#endif
+                continue;
             }
+
+            if (layer->getValueExtents(min, max, log, unit)) {
+                scaleLayer = layer;
+
+#ifdef DEBUG_PANE_SCALE_CHOICE
+                SVCERR << "Pane[" << getId() << "]::drawVerticalScale: "
+                       << "Layer " << layer
+                       << ", " << layer->getLayerPresentationName()
+                       << " has value extents (unit = "
+                       << unit << "), using this layer or unit" << endl;
+#endif
+                break;
+            }
+
+            if (layer->isLayerOpaque()) {
+#ifdef DEBUG_PANE
+                SVCERR << "Pane[" << getId() << "]::drawVerticalScale: "
+                       << "Layer " << layer
+                       << ", " << layer->getLayerPresentationName()
+                       << " is opaque, searching no further" << endl;
+#endif
+                break;
+            }
+        }
+
+        if (scaleLayer) {
+            scaleWidth = scaleLayer->getVerticalScaleWidth
+                (this, includeColourScale, paint);
+
+#ifdef DEBUG_PANE_SCALE_CHOICE
+            SVCERR << "Pane[" << getId() << "]::drawVerticalScale: Layer "
+                   << scaleLayer << ", "
+                   << scaleLayer->getLayerPresentationName()
+                   << " offers vertical scale width of "
+                   << scaleWidth << endl;
+#endif
+        }
+        
+        if (scaleWidth == 0 && unit != "") {
+#ifdef DEBUG_PANE_SCALE_CHOICE
+            SVDEBUG << "Pane[" << getId()
+                    << "]::drawVerticalScale: No good scale layer, "
+                    << "but we have a unit of " << unit
+                    << " - seeking scale-providing layer for that" << endl;
+#endif
+            
+            scaleLayer = getScaleProvidingLayerForUnit(unit);
+            
+#ifdef DEBUG_PANE_SCALE_CHOICE
+            SVDEBUG << "Pane[" << getId()
+                    << "]::drawVerticalScale: That returned layer "
+                    << scaleLayer << ", "
+                    << (scaleLayer ? scaleLayer->getLayerPresentationName()
+                        : "(none)")
+                    <<  endl;
+#endif
         }
     }
 
-    if (!scaleLayer) m_scaleWidth = 0;
-
-//    cerr << "m_scaleWidth = " << m_scaleWidth << ", r.left = " << r.left() << endl;
-    
+    if (scaleWidth > 0) {
+        m_scaleWidth = scaleWidth;
+    } else if (scaleLayer) {
+        m_scaleWidth = scaleLayer->getVerticalScaleWidth
+            (this, includeColourScale, paint);
+    } else {
+        m_scaleWidth = 0;
+    }
+        
     if (m_scaleWidth > 0 && r.left() < m_scaleWidth) {
 
 //      Profiler profiler("Pane::paintEvent - painting vertical scale", true);
 
-//      SVDEBUG << "Pane::paintEvent: calling paint.save() in vertical scale block" << endl;
         paint.save();
             
         paint.setPen(Qt::NoPen);
@@ -613,8 +672,8 @@ Pane::drawVerticalScale(QRect r, Layer *topLayer, QPainter &paint)
 
         paint.setBrush(Qt::NoBrush);
         scaleLayer->paintVerticalScale
-            (this, m_manager->shouldShowVerticalColourScale(),
-             paint, QRect(0, 0, m_scaleWidth, height()));
+            (this, includeColourScale, paint,
+             QRect(0, 0, m_scaleWidth, height()));
         
         paint.restore();
     }
@@ -630,6 +689,11 @@ Pane::drawFeatureDescription(Layer *topLayer, QPainter &paint)
         
         paint.save();
         
+    // Qt 5.13 deprecates QFontMetrics::width(), but its suggested
+    // replacement (horizontalAdvance) was only added in Qt 5.11
+    // which is too new for us
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
         int tabStop =
             paint.fontMetrics().width(tr("Some lengthy prefix:"));
         
@@ -748,8 +812,11 @@ Pane::drawCentreLine(sv_samplerate_t sampleRate, QPainter &paint, bool omitLine)
 }
 
 void
-Pane::drawModelTimeExtents(QRect r, QPainter &paint, const Model *model)
+Pane::drawModelTimeExtents(QRect r, QPainter &paint, ModelId modelId)
 {
+    auto model = ModelById::get(modelId);
+    if (!model) return;
+
     paint.save();
     
     QBrush brush;
@@ -786,14 +853,17 @@ Pane::drawModelTimeExtents(QRect r, QPainter &paint, const Model *model)
 }
 
 void
-Pane::drawAlignmentStatus(QRect r, QPainter &paint, const Model *model,
+Pane::drawAlignmentStatus(QRect r, QPainter &paint, ModelId modelId,
                           bool down)
 {
-    const Model *reference = model->getAlignmentReference();
+    auto model = ModelById::get(modelId);
+    if (!model) return;
+    
+    ModelId reference = model->getAlignmentReference();
 /*
     if (!reference) {
         cerr << "Pane[" << this << "]::drawAlignmentStatus: No reference" << endl;
-    } else if (reference == model) {
+    } else if (reference == model->getId()) {
         cerr << "Pane[" << this << "]::drawAlignmentStatus: This is the reference model" << endl;
     } else {
         cerr << "Pane[" << this << "]::drawAlignmentStatus: This is not the reference" << endl;
@@ -802,16 +872,25 @@ Pane::drawAlignmentStatus(QRect r, QPainter &paint, const Model *model,
     QString text;
     int completion = 100;
 
-    if (reference == model) {
+    if (reference == modelId) {
         text = tr("Reference");
-    } else if (!reference) {
+    } else if (reference.isNone()) {
         text = tr("Unaligned");
     } else {
         completion = model->getAlignmentCompletion();
+        int relativePitch = 0;
+        if (auto alignmentModel =
+            ModelById::getAs<AlignmentModel>(model->getAlignment())) {
+            relativePitch = alignmentModel->getRelativePitch();
+        }
         if (completion == 0) {
             text = tr("Unaligned");
         } else if (completion < 100) {
             text = tr("Aligning: %1%").arg(completion);
+        } else if (relativePitch < 0) {
+            text = tr("Aligned at -%1 cents").arg(-relativePitch);
+        } else if (relativePitch > 0) {
+            text = tr("Aligned at +%1 cents").arg(relativePitch);
         } else {
             text = tr("Aligned");
         }
@@ -839,15 +918,18 @@ Pane::drawAlignmentStatus(QRect r, QPainter &paint, const Model *model,
 }
 
 void
-Pane::modelAlignmentCompletionChanged()
+Pane::modelAlignmentCompletionChanged(ModelId modelId)
 {
-    View::modelAlignmentCompletionChanged();
+    View::modelAlignmentCompletionChanged(modelId);
     update(QRect(0, 0, 300, 100));
 }
 
 void
-Pane::drawWorkTitle(QRect r, QPainter &paint, const Model *model)
+Pane::drawWorkTitle(QRect r, QPainter &paint, ModelId modelId)
 {
+    auto model = ModelById::get(modelId);
+    if (!model) return;
+    
     QString title = model->getTitle();
     QString maker = model->getMaker();
 //SVDEBUG << "Pane::drawWorkTitle: title=\"" << title//<< "\", maker=\"" << maker << "\"" << endl;
@@ -1023,9 +1105,12 @@ Pane::drawEditingSelection(QPainter &paint)
 }
 
 void
-Pane::drawDurationAndRate(QRect r, const Model *waveformModel,
+Pane::drawDurationAndRate(QRect r, ModelId waveformModelId,
                           sv_samplerate_t sampleRate, QPainter &paint)
 {
+    auto waveformModel = ModelById::get(waveformModelId);
+    if (!waveformModel) return;
+    
     int fontHeight = paint.fontMetrics().height();
     int fontAscent = paint.fontMetrics().ascent();
 
@@ -1131,7 +1216,7 @@ Pane::renderPartToNewImage(sv_frame_t f0, sv_frame_t f1)
     if (!render(*paint, 0, f0, f1)) {
         delete paint;
         delete image;
-        return 0;
+        return nullptr;
     } else {
         delete paint;
         return image;
@@ -1267,7 +1352,7 @@ Pane::getTopFlexiNoteLayer()
             return m_layerStack[i];
         }
     }
-    return 0;
+    return nullptr;
 }
 
 void
@@ -1355,7 +1440,7 @@ Pane::mousePressEvent(QMouseEvent *e)
             if (layer && !m_shiftPressed &&
                 !qobject_cast<TimeRulerLayer *>(layer)) { // don't snap to secs
                 layer->snapToFeatureFrame(this, snapFrame,
-                                          resolution, Layer::SnapLeft);
+                                          resolution, Layer::SnapLeft, e->y());
             }
         
             if (snapFrame < 0) snapFrame = 0;
@@ -1443,7 +1528,9 @@ Pane::mouseReleaseEvent(QMouseEvent *e)
         return;
     }
 
-//    cerr << "mouseReleaseEvent" << endl;
+#ifdef DEBUG_PANE
+    SVCERR << "Pane[" << getId() << "]::mouseReleaseEvent" << endl;
+#endif
 
     ViewManager::ToolMode mode = ViewManager::NavigateMode;
     if (m_manager) mode = m_manager->getToolModeFor(this);
@@ -1869,7 +1956,7 @@ Pane::zoomToRegion(QRect r)
     QString unit;
     double min, max;
     bool log;
-    Layer *layer = 0;
+    Layer *layer = nullptr;
     for (LayerList::const_iterator i = m_layerStack.begin();
          i != m_layerStack.end(); ++i) { 
         if ((*i)->getValueExtents(min, max, log, unit) &&
@@ -1938,9 +2025,11 @@ Pane::dragTopLayer(QMouseEvent *e)
     if (m_dragMode == HorizontalDrag ||
         m_dragMode == FreeDrag) {
 
-        sv_frame_t frameOff = getFrameForX(e->x()) - getFrameForX(m_clickPos.x());
+        sv_frame_t fromFrame = getFrameForX(m_clickPos.x());
+        sv_frame_t toFrame = getFrameForX(e->x());
+        sv_frame_t frameOff = toFrame - fromFrame;
+
         sv_frame_t newCentreFrame = m_dragCentreFrame;
-        
         if (frameOff < 0) {
             newCentreFrame -= frameOff;
         } else if (newCentreFrame >= frameOff) {
@@ -1949,9 +2038,15 @@ Pane::dragTopLayer(QMouseEvent *e)
             newCentreFrame = 0;
         }
 
-#ifdef DEBUG_PANE       
-        SVDEBUG << "Pane::dragTopLayer: newCentreFrame = " << newCentreFrame <<
-            ", models end frame = " << getModelsEndFrame() << endl;
+#ifdef DEBUG_PANE
+        SVDEBUG << "Pane::dragTopLayer: dragged from x = "
+                << m_clickPos.x() << " to " << e->x()
+                << ", from frame = " << fromFrame
+                << " to " << toFrame
+                << ", for frame offset of " << frameOff << endl;
+        SVDEBUG << "Pane::dragTopLayer: newCentreFrame = " << newCentreFrame
+                << ", dragCentreFrame = " << m_dragCentreFrame
+                << ", models end frame = " << getModelsEndFrame() << endl;
 #endif
 
         if (newCentreFrame >= getModelsEndFrame()) {
@@ -2078,9 +2173,9 @@ Pane::dragExtendSelection(QMouseEvent *e)
     if (layer && !m_shiftPressed &&
         !qobject_cast<TimeRulerLayer *>(layer)) { // don't snap to secs
         layer->snapToFeatureFrame(this, snapFrameLeft,
-                                  resolution, Layer::SnapLeft);
+                                  resolution, Layer::SnapLeft, e->y());
         layer->snapToFeatureFrame(this, snapFrameRight,
-                                  resolution, Layer::SnapRight);
+                                  resolution, Layer::SnapRight, e->y());
     }
         
 //        cerr << "snap: frame = " << mouseFrame << ", start frame = " << m_selectionStartFrame << ", left = " << snapFrameLeft << ", right = " << snapFrameRight << endl;
@@ -2187,8 +2282,9 @@ Pane::mouseDoubleClickEvent(QMouseEvent *e)
         return;
     }
 
-    if (mode == ViewManager::NavigateMode ||
-        mode == ViewManager::EditMode) {
+    if (mode == ViewManager::EditMode ||
+        (mode == ViewManager::NavigateMode &&
+         m_manager->getOpportunisticEditingEnabled())) {
 
         Layer *layer = getInteractionLayer();
         if (layer && layer->isLayerEditable()) {
@@ -2298,11 +2394,19 @@ Pane::wheelEvent(QWheelEvent *e)
         // Coarse wheel information (or vertical zoom, which is
         // necessarily coarse itself)
 
-        // Sometimes on Linux we're seeing absurdly extreme angles on
-        // the first wheel event -- discard those entirely
-        if (abs(m_pendingWheelAngle) >= 600) {
-            m_pendingWheelAngle = 0;
-            return;
+        // Sometimes on Linux we're seeing very extreme angles on the
+        // first wheel event. They could be spurious, or they could be
+        // a result of the user frantically wheeling away while the
+        // pane was unresponsive for some reason. We don't want to
+        // discard them, as that makes the application feel even less
+        // responsive, but if we take them literally we risk changing
+        // the view so radically that the user won't recognise what
+        // has happened. Clamp them instead.
+        if (m_pendingWheelAngle > 600) {
+            m_pendingWheelAngle = 600;
+        }
+        if (m_pendingWheelAngle < -600) {
+            m_pendingWheelAngle = -600;
         }
 
         while (abs(m_pendingWheelAngle) >= 120) {
@@ -2405,7 +2509,7 @@ Pane::horizontalThumbwheelMoved(int value)
 void
 Pane::verticalThumbwheelMoved(int value)
 {
-    Layer *layer = 0;
+    Layer *layer = nullptr;
     if (getLayerCount() > 0) layer = getLayer(getLayerCount() - 1);
     if (layer) {
         int defaultStep = 0;
@@ -2681,7 +2785,7 @@ Pane::viewZoomLevelChanged(View *v, ZoomLevel z, bool locked)
 void
 Pane::propertyContainerSelected(View *v, PropertyContainer *pc)
 {
-    Layer *layer = 0;
+    Layer *layer = nullptr;
 
     if (getLayerCount() > 0) {
         layer = getLayer(getLayerCount() - 1);
@@ -2693,7 +2797,7 @@ Pane::propertyContainerSelected(View *v, PropertyContainer *pc)
     updateHeadsUpDisplay();
 
     if (m_vthumb) {
-        RangeMapper *rm = 0;
+        RangeMapper *rm = nullptr;
         if (layer) rm = layer->getNewVerticalZoomRangeMapper();
         if (rm) m_vthumb->setRangeMapper(rm);
     }
@@ -2708,7 +2812,7 @@ Pane::propertyContainerSelected(View *v, PropertyContainer *pc)
 void
 Pane::verticalZoomChanged()
 {
-    Layer *layer = 0;
+    Layer *layer = nullptr;
 
     if (getLayerCount() > 0) {
 
