@@ -57,11 +57,11 @@
 
 PropertyBox::PropertyBox(PropertyContainer *container) :
     m_container(container),
-    m_showButton(0),
-    m_playButton(0)
+    m_showButton(nullptr),
+    m_playButton(nullptr)
 {
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this << "(\"" <<
+    SVDEBUG << "PropertyBox[" << this << "(\"" <<
         container->getPropertyContainerName() << "\" at " << container << ")]::PropertyBox" << endl;
 #endif
 
@@ -82,7 +82,7 @@ PropertyBox::PropertyBox(PropertyContainer *container) :
     m_mainBox->addWidget(m_mainWidget);
     m_mainBox->insertStretch(2, 10);
 
-    m_viewPlayFrame = 0;
+    m_viewPlayFrame = nullptr;
     populateViewPlayFrame();
 
     m_layout = new QGridLayout;
@@ -109,14 +109,14 @@ PropertyBox::PropertyBox(PropertyContainer *container) :
             this, SLOT(unitDatabaseChanged()));
 
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this << "]::PropertyBox returning" << endl;
+    SVDEBUG << "PropertyBox[" << this << "]::PropertyBox returning" << endl;
 #endif
 }
 
 PropertyBox::~PropertyBox()
 {
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this << "]::~PropertyBox" << endl;
+    SVDEBUG << "PropertyBox[" << this << "]::~PropertyBox" << endl;
 #endif
 }
 
@@ -124,12 +124,12 @@ void
 PropertyBox::populateViewPlayFrame()
 {
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this << ":" << m_container << "]::populateViewPlayFrame" << endl;
+    SVDEBUG << "PropertyBox[" << this << ":" << m_container << "]::populateViewPlayFrame" << endl;
 #endif
 
     if (m_viewPlayFrame) {
         delete m_viewPlayFrame;
-        m_viewPlayFrame = 0;
+        m_viewPlayFrame = nullptr;
     }
 
     if (!m_container) return;
@@ -142,7 +142,7 @@ PropertyBox::populateViewPlayFrame()
                 this, SLOT(populateViewPlayFrame()));
     }
 
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params && !layer) return;
 
     m_viewPlayFrame = new QFrame;
@@ -175,7 +175,7 @@ PropertyBox::populateViewPlayFrame()
                 this, SLOT(mouseEnteredWidget()));
         connect(m_playButton, SIGNAL(mouseLeft()),
                 this, SLOT(mouseLeftWidget()));
-        connect(params, SIGNAL(playAudibleChanged(bool)),
+        connect(params.get(), SIGNAL(playAudibleChanged(bool)),
                 this, SLOT(playAudibleChanged(bool)));
 
         LevelPanToolButton *levelPan = new LevelPanToolButton;
@@ -186,9 +186,9 @@ PropertyBox::populateViewPlayFrame()
                 this, SLOT(playGainControlChanged(float)));
         connect(levelPan, SIGNAL(panChanged(float)),
                 this, SLOT(playPanControlChanged(float)));
-        connect(params, SIGNAL(playGainChanged(float)),
+        connect(params.get(), SIGNAL(playGainChanged(float)),
                 levelPan, SLOT(setLevel(float)));
-        connect(params, SIGNAL(playPanChanged(float)),
+        connect(params.get(), SIGNAL(playPanChanged(float)),
                 levelPan, SLOT(setPan(float)));
         connect(levelPan, SIGNAL(mouseEntered()),
                 this, SLOT(mouseEnteredWidget()));
@@ -248,12 +248,12 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
     QString iconName = m_container->getPropertyIconName(name);
 
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this
+    SVDEBUG << "PropertyBox[" << this
               << "(\"" << m_container->getPropertyContainerName()
               << "\")]";
-    cerr << "::updatePropertyEditor(\"" << name << "\", "
+    SVDEBUG << "::updatePropertyEditor(\"" << name << "\", "
          << rangeChanged << "):";
-    cerr << " type " << type << ", value " << value
+    SVDEBUG << " type " << type << ", value " << value
          << ", have " << have << ", group \"" << groupName << "\"" << endl;
 #endif
 
@@ -287,7 +287,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
     QGridLayout *groupLayout = m_groupLayouts[groupName];
 
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "groupName becomes \"" << groupName << "\", groupLabel = \""
+    SVDEBUG << "groupName becomes \"" << groupName << "\", groupLabel = \""
          << groupLabel << "\", groupLayout = " << groupLayout << endl;
 #endif
     
@@ -303,7 +303,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
 
         if (!(button = qobject_cast<QAbstractButton *>(existing))) {
 #ifdef DEBUG_PROPERTY_BOX 
-            cerr << "PropertyBox: creating new checkbox" << endl;
+            SVDEBUG << "PropertyBox: creating new checkbox" << endl;
 #endif
             if (iconName != "") {
 #ifdef Q_OS_MAC
@@ -360,7 +360,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
             }
         } else {
 #ifdef DEBUG_PROPERTY_BOX 
-            cerr << "PropertyBox: creating new dial" << endl;
+            SVDEBUG << "PropertyBox: creating new dial" << endl;
 #endif
             dial = new AudioDial();
             dial->setObjectName(name);
@@ -408,7 +408,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
         if (!(cb = qobject_cast<ColourComboBox *>(existing))) {
 
 #ifdef DEBUG_PROPERTY_BOX 
-            cerr << "PropertyBox: creating new colour combobox" << endl;
+            SVDEBUG << "PropertyBox: creating new colour combobox" << endl;
 #endif
             cb = new ColourComboBox(true);
             cb->setObjectName(name);
@@ -447,7 +447,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
 
         if (!(cb = qobject_cast<ColourMapComboBox *>(existing))) {
 #ifdef DEBUG_PROPERTY_BOX 
-            cerr << "PropertyBox: creating new colourmap combobox" << endl;
+            SVDEBUG << "PropertyBox: creating new colourmap combobox" << endl;
 #endif
             cb = new ColourMapComboBox(false);
             cb->setObjectName(name);
@@ -487,7 +487,7 @@ PropertyBox::updatePropertyEditor(PropertyContainer::PropertyName name,
 
         if (!(cb = qobject_cast<NotifyingComboBox *>(existing))) {
 #ifdef DEBUG_PROPERTY_BOX 
-            cerr << "PropertyBox: creating new combobox" << endl;
+            SVDEBUG << "PropertyBox: creating new combobox" << endl;
 #endif
             cb = new NotifyingComboBox();
             cb->setObjectName(name);
@@ -607,13 +607,13 @@ void
 PropertyBox::unitDatabaseChanged()
 {
 #ifdef DEBUG_PROPERTY_BOX
-    cerr << "PropertyBox[" << this << "]: unitDatabaseChanged" << endl;
+    SVDEBUG << "PropertyBox[" << this << "]: unitDatabaseChanged" << endl;
 #endif
     blockSignals(true);
 
-//    cerr << "my container is " << m_container << endl;
-//    cerr << "my container's name is... " << endl;
-//    cerr << m_container->objectName() << endl;
+//    SVDEBUG << "my container is " << m_container << endl;
+//    SVDEBUG << "my container's name is... " << endl;
+//    SVDEBUG << m_container->objectName() << endl;
 
     PropertyContainer::PropertyList properties = m_container->getProperties();
     for (size_t i = 0; i < properties.size(); ++i) {
@@ -639,12 +639,13 @@ PropertyBox::propertyControllerChanged(int value)
     QString name = obj->objectName();
 
 #ifdef DEBUG_PROPERTY_BOX
-    SVDEBUG << "PropertyBox::propertyControllerChanged(" << name              << ", " << value << ")" << endl;
+    SVDEBUG << "PropertyBox::propertyControllerChanged(" << name
+            << ", " << value << ")" << endl;
 #endif
     
     PropertyContainer::PropertyType type = m_container->getPropertyType(name);
 
-    Command *c = 0;
+    Command *c = nullptr;
 
     if (type == PropertyContainer::UnitsProperty) {
 
@@ -674,7 +675,7 @@ PropertyBox::playAudibleChanged(bool audible)
 void
 PropertyBox::playAudibleButtonChanged(bool audible)
 {
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params) return;
 
     if (params->isPlayAudible() != audible) {
@@ -690,7 +691,7 @@ PropertyBox::playGainControlChanged(float gain)
 {
     QObject *obj = sender();
 
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params) return;
 
     if (params->getPlayGain() != gain) {
@@ -708,7 +709,7 @@ PropertyBox::playPanControlChanged(float pan)
 {
     QObject *obj = sender();
 
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params) return;
 
     if (params->getPlayPan() != pan) {
@@ -724,7 +725,7 @@ PropertyBox::playPanControlChanged(float pan)
 void
 PropertyBox::editPlayParameters()
 {
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params) return;
 
     QString clip = params->getPlayClipId();
@@ -774,7 +775,7 @@ PropertyBox::editPlayParameters()
 void
 PropertyBox::playClipChanged(QString id)
 {
-    PlayParameters *params = m_container->getPlayParameters();
+    auto params = m_container->getPlayParameters();
     if (!params) return;
 
     params->setPlayClipId(id);
@@ -823,7 +824,7 @@ PropertyBox::updateContextHelp(QObject *o)
 
     } else if (wname == "playParamButton") {
  
-        PlayParameters *params = m_container->getPlayParameters();
+        auto params = m_container->getPlayParameters();
         if (params) {
             help = tr("Change sound used for playback (currently \"%1\")")
                 .arg(params->getPlayClipId());
